from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from . import models


class AudioPlayerPlugin(CMSPluginBase):
    model = models.AudioPlayer
    name = _('Audio player')
    allow_children = True
    child_classes = ['AudioFilePlugin', 'AudioFolderPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'template',
                'label',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        })
    ]

    def render(self, context, instance, placeholder):
        context = super().render(context, instance, placeholder)
        context['audio_template'] = instance.template
        return context

    def get_render_template(self, context, instance, placeholder):
        return 'djangocms_audio/{}/audio_player.html'.format(instance.template)


class AudioFilePlugin(CMSPluginBase):
    model = models.AudioFile
    name = _('File')
    module = _('Audio player')
    allow_children = True
    require_parent = True
    parent_classes = ['AudioPlayerPlugin']
    child_classes = ['AudioTrackPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'audio_file',
                'text_title',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'text_description',
                'attributes',
            )
        })
    ]

    def get_render_template(self, context, instance, placeholder):
        return 'djangocms_audio/{}/file.html'.format(context['audio_template'])


class AudioFolderPlugin(CMSPluginBase):
    model = models.AudioFolder
    name = _('Folder')
    module = _('Audio player')
    require_parent = True
    parent_classes = ['AudioPlayerPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'audio_folder',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        })
    ]

    def get_render_template(self, context, instance, placeholder):
        return 'djangocms_audio/{}/folder.html'.format(context['audio_template'])


class AudioTrackPlugin(CMSPluginBase):
    model = models.AudioTrack
    name = _('Track')
    module = _('Audio player')
    require_parent = True
    parent_classes = ['AudioFilePlugin']

    fieldsets = [
        (None, {
            'fields': (
                'kind',
                'src',
                'srclang',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'label',
                'attributes',
            )
        })
    ]

    def get_render_template(self, context, instance, placeholder):
        return 'djangocms_audio/{}/track.html'.format(context['audio_template'])


plugin_pool.register_plugin(AudioPlayerPlugin)
plugin_pool.register_plugin(AudioFilePlugin)
plugin_pool.register_plugin(AudioFolderPlugin)
plugin_pool.register_plugin(AudioTrackPlugin)



from cms.api import add_plugin, create_page
from cms.test_utils.testcases import CMSTestCase

from djangocms_audio.cms_plugins import (
    AudioFilePlugin, AudioFolderPlugin, AudioPlayerPlugin, AudioTrackPlugin,
)

from .helpers import get_filer_file, get_filer_folder


class AudioPlayerPluginsTestCase(CMSTestCase):

    def setUp(self):
        self.language = "en"
        self.home = create_page(
            title='home',
            template='page.html',
            language=self.language,
        )
        self.home.publish(self.language)
        self.page = create_page(
            title='content',
            template='page.html',
            language=self.language,
        )
        self.page.publish(self.language)
        self.placeholder = self.page.placeholders.get(slot="content")
        self.superuser = self.get_superuser()
        self.audio_file = get_filer_file("test_file.mp3")
        self.track_file = get_filer_file("test_track.vtt")

    def tearDown(self):
        self.page.delete()
        self.home.delete()
        self.superuser.delete()
        self.audio_file.delete()
        self.track_file.delete()

    def test_player_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AudioPlayerPlugin.__name__,
            language=self.language,
        )
        self.assertEqual(plugin.plugin_type, "AudioPlayerPlugin")

    def test_file_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AudioFilePlugin.__name__,
            language=self.language,
        )
        self.assertEqual(plugin.plugin_type, "AudioFilePlugin")

    def test_folder_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AudioFolderPlugin.__name__,
            language=self.language,
        )
        self.assertEqual(plugin.plugin_type, "AudioFolderPlugin")

    def test_track_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AudioTrackPlugin.__name__,
            language=self.language,
        )
        self.assertEqual(plugin.plugin_type, "AudioTrackPlugin")

    def test_plugin_structure(self):
        request_url = self.page.get_absolute_url(self.language) + "?toolbar_off=true"

        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AudioPlayerPlugin.__name__,
            language=self.language,
            template="default",
        )
        self.page.publish(self.language)
        self.assertEqual(parent.get_plugin_class_instance().name, "Audio player")

        with self.login_user_context(self.superuser):
            response = self.client.get(request_url)

        self.assertIn(b"No audio file available.", response.content)

        child = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=AudioFilePlugin.__name__,
            language=self.language,
            audio_file=self.audio_file,
        )
        self.page.publish(self.language)
        self.assertEqual(child.audio_file.label, "test_file.mp3")

        with self.login_user_context(self.superuser):
            response = self.client.get(request_url)

        self.assertIn(b"<audio controls", response.content)
        self.assertNotIn(b"No audio file available.", response.content)
        self.assertContains(response, self.audio_file.label)

        track = add_plugin(
            target=child,
            placeholder=self.placeholder,
            plugin_type=AudioTrackPlugin.__name__,
            language=self.language,
            kind="subtitles",
            src=self.track_file,
            srclang=self.language,
        )
        self.page.publish(self.language)
        self.assertEqual(track.src.label, "test_track.vtt")

        with self.login_user_context(self.superuser):
            response = self.client.get(request_url)

        self.assertIn(b"<track kind", response.content)
        self.assertNotIn(b"No audio file available.", response.content)
        self.assertContains(response, self.track_file.label)

        folder = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=AudioFolderPlugin.__name__,
            language=self.language,
            audio_folder=get_filer_folder(),
        )
        self.page.publish(self.language)
        self.assertEqual(folder.audio_folder.name, "test_folder")

        with self.login_user_context(self.superuser):
            response = self.client.get(request_url)

        self.assertContains(response, "No matching audio files were found in the specified folder.")



from django.db import migrations, models

from djangocms_audio.models import get_templates


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_audio', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='audiofile',
            name='text_title',
            field=models.CharField(max_length=255, verbose_name='Title', blank=True),
        ),
        migrations.AlterField(
            model_name='audioplayer',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='audioplayer',
            name='template',
            field=models.CharField(default=get_templates()[0][0], max_length=255, verbose_name='Template', choices=get_templates()),
        ),
        migrations.AlterField(
            model_name='audiotrack',
            name='kind',
            field=models.CharField(max_length=255, verbose_name='Kind', choices=[('subtitles', 'Subtitles'), ('captions', 'Captions'), ('descriptions', 'Descriptions'), ('chapters', 'Chapters')]),
        ),
        migrations.AlterField(
            model_name='audiotrack',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='audiotrack',
            name='srclang',
            field=models.CharField(help_text='Examples: "en" or "de" etc.', max_length=255, verbose_name='Source language', blank=True),
        ),
    ]



#!/usr/bin/env python
HELPER_SETTINGS = {
    'INSTALLED_APPS': [
        'easy_thumbnails',
        'filer',
        'mptt',
    ],
    'CMS_LANGUAGES': {
        1: [{
            'code': 'en',
            'name': 'English',
        }]
    },
    'LANGUAGE_CODE': 'en',
    'ALLOWED_HOSTS': ['localhost'],
}


def run():
    from app_helper import runner
    runner.cms('djangocms_audio')


if __name__ == '__main__':
    run()



import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields
import filer.fields.folder


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0006_auto_20160623_1627'),
    ]

    operations = [
        migrations.CreateModel(
            name='AudioFile',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, related_name='djangocms_audio_audiofile', auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('text_title', models.CharField(max_length=200, verbose_name='Title', blank=True)),
                ('text_description', models.TextField(verbose_name='Description', blank=True)),
                ('attributes', djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True)),
                ('audio_file', filer.fields.file.FilerFileField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='File', to='filer.File', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AudioFolder',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, related_name='djangocms_audio_audiofolder', auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(default=dict, help_text='Is applied to all audio file instances.', verbose_name='Attributes', blank=True)),
                ('audio_folder', filer.fields.folder.FilerFolderField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='Folder', to='filer.Folder', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AudioPlayer',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, related_name='djangocms_audio_audioplayer', auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(default='default', max_length=50, verbose_name='Template', choices=[('default', 'Default')])),
                ('label', models.CharField(max_length=200, verbose_name='Label', blank=True)),
                ('attributes', djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AudioTrack',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, related_name='djangocms_audio_audiotrack', auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('kind', models.CharField(max_length=50, verbose_name='Kind', choices=[('subtitles', 'Subtitles'), ('captions', 'Captions'), ('descriptions', 'Descriptions'), ('chapters', 'Chapters')])),
                ('srclang', models.CharField(help_text='Examples: "en" or "de" etc.', max_length=10, verbose_name='Source language', blank=True)),
                ('label', models.CharField(max_length=200, verbose_name='Label', blank=True)),
                ('attributes', djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True)),
                ('src', filer.fields.file.FilerFileField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='Source file', to='filer.File', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



"""
Enables the user to add an "Audio player" plugin that serves as
a wrapper rendering the player and its options.

The "Audio player" plugin allows to add either a single "File" or a reference
to a "Folder" as children.
"""
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from cms.models import CMSPlugin

from djangocms_attributes_field.fields import AttributesField
from filer.fields.file import FilerFileField
from filer.fields.folder import FilerFolderField


# mp3 is supported by all major browsers
def get_extensions():
    extensions = getattr(
        settings,
        'DJANGOCMS_AUDIO_ALLOWED_EXTENSIONS',
        ['mp3', 'ogg'],
    )
    return extensions


# Add additional choices through the ``settings.py``.
def get_templates():
    choices = [
        ('default', _('Default')),
    ]
    choices += getattr(
        settings,
        'DJANGOCMS_AUDIO_TEMPLATES',
        [],
    )
    return choices


class AudioPlayer(CMSPlugin):
    """
    Renders a container around the HTML <audio> elements.
    """
    template = models.CharField(
        verbose_name=_('Template'),
        choices=get_templates(),
        default=get_templates()[0][0],
        max_length=255,
    )
    label = models.CharField(
        verbose_name=_('Label'),
        blank=True,
        max_length=255,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )

    def __str__(self):
        return self.label or str(self.pk)


class AudioFile(CMSPlugin):
    """
    Renders the HTML <audio> element, add params through attributes.
    """
    audio_file = FilerFileField(
        verbose_name=_('File'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    text_title = models.CharField(
        verbose_name=_('Title'),
        blank=True,
        max_length=255,
    )
    text_description = models.TextField(
        verbose_name=_('Description'),
        blank=True,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )

    def __str__(self):
        if self.audio_file_id and self.audio_file.label:
            return self.audio_file.label
        return str(self.pk)

    def clean(self):
        if self.audio_file and self.audio_file.extension not in get_extensions():
            raise ValidationError(
                gettext('Incorrect file type: {extension}.').format(
                    extension=self.audio_file.extension)
            )

    def get_short_description(self):
        if self.audio_file_id and self.audio_file.label:
            return self.audio_file.label
        return gettext('<file is missing>')

    def copy_relations(self, oldinstance):
        # Because we have a ForeignKey, it's required to copy over
        # the reference from the instance to the new plugin.
        self.audio_file = oldinstance.audio_file


class AudioFolder(CMSPlugin):
    """
    Render files contained in a folder, only get_extensions() are considered.
    If you desire more customisation (title name, description) use the
    File plugin.
    """
    audio_folder = FilerFolderField(
        verbose_name=_('Folder'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
        help_text=_('Is applied to all audio file instances.'),
    )

    def __str__(self):
        if self.audio_folder_id and self.audio_folder.name:
            return self.audio_folder.name
        return str(self.pk)

    def get_files(self):
        files = []

        for audio_file in self.audio_folder.files:
            if audio_file.extension in get_extensions():
                files.append(audio_file)
        return files

    def get_short_description(self):
        if self.audio_folder_id and self.audio_folder.name:
            return self.audio_folder.name
        return gettext('<folder is missing>')

    def copy_relations(self, oldinstance):
        # Because we have a ForeignKey, it's required to copy over
        # the reference from the instance to the new plugin.
        self.audio_folder = oldinstance.audio_folder


class AudioTrack(CMSPlugin):
    """
    Renders the HTML <track> element inside <audio>.
    """
    KIND_CHOICES = [
        ('subtitles', _('Subtitles')),
        ('captions', _('Captions')),
        ('descriptions', _('Descriptions')),
        ('chapters', _('Chapters')),
    ]

    kind = models.CharField(
        verbose_name=_('Kind'),
        choices=KIND_CHOICES,
        max_length=255,
    )
    src = FilerFileField(
        verbose_name=_('Source file'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    srclang = models.CharField(
        verbose_name=_('Source language'),
        blank=True,
        max_length=255,
        help_text=_('Examples: "en" or "de" etc.'),
    )
    label = models.CharField(
        verbose_name=_('Label'),
        blank=True,
        max_length=255,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )

    def __str__(self):
        label = self.kind
        if self.srclang:
            label += ' ({})'.format(self.srclang)
        return label



from django.conf import settings
from django.core.exceptions import ValidationError
from django.test import TestCase

from djangocms_audio.models import (
    AudioFile, AudioFolder, AudioPlayer, AudioTrack, get_extensions,
    get_templates,
)

from .helpers import get_filer_file, get_filer_folder


class AudioPlayerModelsTestCase(TestCase):

    def setUp(self):
        self.audio = get_filer_file("test_file.mp3")
        self.folder = get_filer_folder()

    def tearDown(self):
        if self.audio:
            self.audio.delete()
            del self.audio
            with self.assertRaises(AttributeError):
                print(self.audio)
        if self.folder:
            self.folder.delete()
            del self.folder
            with self.assertRaises(AttributeError):
                print(self.folder)

        AudioPlayer.objects.filter(pk=1).delete()
        self.assertEqual(len(AudioPlayer.objects.all()), 0)
        AudioFile.objects.filter(pk=1).delete()
        self.assertEqual(len(AudioFile.objects.all()), 0)
        AudioFolder.objects.filter(pk=1).delete()
        self.assertEqual(len(AudioFolder.objects.all()), 0)
        AudioTrack.objects.filter(pk=1).delete()
        self.assertEqual(len(AudioTrack.objects.all()), 0)

    def test_settings(self):
        self.assertEqual(get_extensions(), ['mp3', 'ogg'])
        settings.DJANGOCMS_AUDIO_ALLOWED_EXTENSIONS = ["mp3", "wav", "flac"]
        self.assertEqual(get_extensions(), ["mp3", "wav", "flac"])

        self.assertEqual(get_templates(), [('default', 'Default')])
        settings.DJANGOCMS_AUDIO_TEMPLATES = [('feature', 'Feature')]
        self.assertEqual(get_templates(), [('default', 'Default'), ('feature', 'Feature')])

    def test_audio_player_instance(self):
        AudioPlayer.objects.create(
            template="default",
            label="audio",
            attributes="{'data-type, 'audio'}"
        )
        instance = AudioPlayer.objects.all()
        self.assertEqual(len(instance), 1)
        instance = AudioPlayer.objects.get(pk=1)
        self.assertEqual(instance.template, "default")
        self.assertEqual(instance.label, "audio")
        self.assertEqual(instance.attributes, "{'data-type, 'audio'}")
        self.assertEqual(instance.__str__(), "audio")
        instance.label = None
        self.assertEqual(instance.__str__(), "1")

    def test_audio_file_instance(self):
        AudioFile.objects.create(
            audio_file=self.audio,
            text_title="sample title",
            text_description="sample description",
            attributes="{'data-type, 'audio'}",
        )
        instance = AudioFile.objects.all()
        self.assertEqual(len(instance), 1)
        instance = AudioFile.objects.get(pk=1)
        self.assertEqual(instance.audio_file, self.audio)
        self.assertEqual(instance.text_title, "sample title")
        self.assertEqual(instance.text_description, "sample description")
        self.assertEqual(instance.attributes, "{'data-type, 'audio'}")
        self.assertEqual(instance.__str__(), "test_file.mp3")
        self.assertEqual(instance.audio_file.label, "test_file.mp3")
        self.assertEqual(instance.audio_file_id, 1)
        self.assertEqual(instance.clean(), None)
        self.assertEqual(instance.get_short_description(), "test_file.mp3")
        # test not allowed extension
        instance.audio_file = get_filer_file("test_file.mp4")
        with self.assertRaises(ValidationError):
            instance.clean()
        instance.audio_file = instance.audio_file.delete()
        # case when the file has been removed
        self.assertEqual(instance.__str__(), "1")
        self.assertEqual(instance.clean(), None)
        self.assertEqual(instance.get_short_description(), "<file is missing>")
        # old copy relation
        instance.copy_relations(instance)
        self.assertEqual(instance.audio_file, None)

    def test_audio_folder_instance(self):
        AudioFolder.objects.create(
            audio_folder=self.folder,
            attributes="{'data-type, 'audio'}",
        )
        instance = AudioFolder.objects.all()
        self.assertEqual(len(instance), 1)
        instance = AudioFolder.objects.get(pk=1)
        self.assertEqual(instance.audio_folder, self.folder)
        self.assertEqual(instance.attributes, "{'data-type, 'audio'}")
        self.assertEqual(instance.__str__(), "test_folder")
        self.assertEqual(instance.get_short_description(), "test_folder")
        # case when the folder has been removed
        instance.audio_folder = None
        self.assertEqual(instance.__str__(), "1")
        self.assertEqual(instance.get_short_description(), "<folder is missing>")
        # old copy relation
        instance.copy_relations(instance)
        self.assertEqual(instance.audio_folder, None)

    def test_audio_folder_instance_files(self):
        self.audio = get_filer_file(
            file_name="test_file.mp3",
            folder=self.folder
        )
        instance = AudioFolder.objects.create(
            audio_folder=self.folder,
        )
        self.assertEqual(instance.get_files(), [self.audio])
        self.audio.delete()
        self.audio = get_filer_file(
            file_name="test_file.mp4",
            folder=self.folder
        )
        self.assertEqual(instance.get_files(), [])

    def test_audio_track_instance(self):
        AudioTrack.objects.create(
            kind=AudioTrack.KIND_CHOICES[0][0],
            src=self.audio,  # should be .vtt normally
            srclang="en",
            label="track label",
            attributes="{'data-type, 'audio'}",
        )
        instance = AudioTrack.objects.all()
        self.assertEqual(len(instance), 1)
        instance = AudioTrack.objects.get(pk=1)
        self.assertEqual(instance.kind, "subtitles")
        self.assertEqual(instance.src, self.audio)
        self.assertEqual(instance.srclang, "en")
        self.assertEqual(instance.label, "track label")
        self.assertEqual(instance.attributes, "{'data-type, 'audio'}")
        self.assertEqual(instance.__str__(), "subtitles (en)")
        # case when the folder has been removed
        instance.srclang = None
        self.assertEqual(instance.__str__(), "subtitles")
